import Taro from '@tarojs/taro'
import { APP_ID, APP_TYPE, APP_VERSION_CODE, VERSION_CODE } from '@/config'

class Http {
  get(url: string, data = {}, config: ConfigType = {}) {
    const { header = {} } = config

    data = wrapData(data)

    return new Promise((success, fail) => {
      const expParams: ConfigType = {
        header: {
          'content-type': 'application/x-www-form-urlencoded',
          ...header
        }
      }

      Taro.request({
        url,
        data,
        success,
        fail,
        ...expParams
      })
    })
  }

  post(url: string, data = {}, config: ConfigType = {}) {
    const { header = {} } = config

    data = wrapData(data)

    if ((data as DataType).baseInfo)
      data = createBaseInfoToFetchData(data)

    return new Promise((resolve, reject) => {
      const expParams: ConfigType = {
        header: {
          'content-type': 'application/x-www-form-urlencoded',
          ...header
        },
        method: 'POST'
      }

      Taro.request({
        url,
        data,
        success(res) {
          const { data, statusCode } = res

          if (statusCode !== 200) {
            reject(data)
            return
          }
          const mockData = {
            "code": 0,
            "msg": "success",
            "data": {
              "ownPlatformAlliance": {
                "adType": "ownPlatformAlliance",
                "platformCreativeInfo": {
                  "id": 3403,
                  "creativeCode": "CREATIVE_17096086444271430",
                  "adId": 1136,
                  "adCode": "AD_17095512471071489",
                  "campaignId": 172,
                  "campaignCode": "CAMPAIGN_17024525411821220",
                  "advertiserId": 332,
                  "advertiserCode": "ADA_17024523941341815",
                  "creativePattern": 3,
                  "creativeTitle": "联盟广告测试",
                  "copywriting": "111",
                  "materialType": "VIDEO",
                  "materialAddress": "http://rescdn.yishihui.com/ad/transcode/prod/video/material_CREATIVE_17096086444271430_1709608644427.mp4",
                  "creativeLogoAddress": "http://rescdn.yishihui.com/ad/prod/image/logo_1709608644242.png",
                  "clickButtonText": "按钮啊啊",
                  "clickButtonColor": "#00C25D",
                  "clickButtonEffects": 0,
                  "landingPageType": 2,
                  "landingPageAddress": "https://nbtga.whjiaoy.com/putin/linkTovnn7Pb?ewpCode=10008&channel=786&subChannel=415&lxqAdid=pqAdOpDrKrMMm8",
                  "positionId": 13,
                  "adpId":"xxxxxx",
                  "trafficCode":"ATP182138123"
                }
              }
            },
            "success": true
          }
          resolve(mockData || data)
        },
        fail(res) {
          reject(res)
        },
        ...expParams
      })
    })
  }
}

export function wrapData(params) {
  const systemInfo = Taro.$global.systemInfo
  const launchOption = Taro.$global.launchOption
  const network = Taro.$global.network

  const defaultInfo = {
    appId: APP_ID,
    appType: APP_TYPE,
    versionCode: VERSION_CODE,
    appVersionCode: APP_VERSION_CODE,
    clientTimestamp: new Date().getTime(),
    platform: systemInfo.platform,
    system: systemInfo.system,
    network: network.networkType,
    careModelStatus: systemInfo.easyMode,
    senceType: launchOption.scene
  }

  return {
    ...defaultInfo,
    ...params
  }
}

export function createBaseInfoToFetchData(data) {
  const baseKey = ['token', 'loginUid', 'appType', 'machineCode', 'platform', 'machineInfo', 'networkType', 'pageSource', 'rootPageSource', 'clientTimestamp', 'sessionId', 'subSessionId', 'requestId', 'pageCategoryId', 'rootPageCategoryId', 'openType', 'shareDepth', 'eventId', 'videoReportMeta', 'rootPageTimestamp', 'returnId', 'system']

  return Object.keys(data).reduce((calc, key) => {
    if (baseKey.includes(key)) {
      calc.baseInfo[key] = data[key]
    } else {
      calc.params[key] = data[key]
    }

    return calc
  }, { baseInfo: {}, params: {} })
}

export default new Http()
